/*

Copyright (c) 2004, Roland J. Weiss.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of the University of Tuebingen nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

import java.io.FileReader;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

import jargs.gnu.CmdLineParser;
 
class pdwrap
{
  
  //----------------------------------------------------------------------------
  // Static Methods.

  /**
  Read the configuration file and append/prepend the prolog/epilog to the
  denoted text file.
  */
  public static void main(String[] args)
  {
    try
    {
      // Parse the command line.
      getArguments(args);

      // Extract prolog and epilog from XML config file.
      SAXParserFactory parser_factory = SAXParserFactory.newInstance();
      SAXParser parser = parser_factory.newSAXParser();
      InputSource in = new InputSource(new FileReader(xmlFileName));
      PDwrapHandler handler = new PDwrapHandler();
      parser.parse(in, handler);

      // Now write prolog + file + epilog to stdout.
      System.out.print(handler.getProlog());
      FileReader tmpFile = new FileReader(tmpFileName);
      int iChar;
      while ((iChar = tmpFile.read()) != -1)
      {
        System.out.print((char) iChar);
      }
      System.out.print(handler.getEpilog());
    }
    catch (Exception e)
    {
      System.out.println("ERROR - Exception caught: "); e.printStackTrace();
    }
  }

  /**
  Parse the command line arguments passed to pdwrap.
  */
  static void getArguments(String[] args)
  {
    CmdLineParser parser = new CmdLineParser();
    CmdLineParser.Option cfgFile = parser.addStringOption('c', "CfgFile");
    CmdLineParser.Option fileName = parser.addStringOption('N', "Name");
    CmdLineParser.Option beginLine = parser.addIntegerOption('b', "begin");
    CmdLineParser.Option endLine = parser.addIntegerOption('e', "end");
    CmdLineParser.Option label = parser.addStringOption('L', "Label");
    CmdLineParser.Option genLabel = parser.addStringOption('D', "GenLabel");
    CmdLineParser.Option genRef = parser.addStringOption('R', "GenRef");
    CmdLineParser.Option caption = parser.addStringOption('A', "Caption");
    CmdLineParser.Option outputType = parser.addStringOption('m', "OutputType");
    CmdLineParser.Option dummy = parser.addBooleanOption('t', "pdweaveCompat");

    try
    {
      parser.parse(args);
    }
    catch (CmdLineParser.OptionException e)
    {
      printUsage();
      System.err.println(e.getMessage());
      System.err.println("Error during command line parsing, exiting.");
      System.exit(2);
    }

    String tmpString;
    tmpString = (String)parser.getOptionValue(cfgFile);
    if (tmpString != null) xmlFileName = tmpString;
    tmpString = (String)parser.getOptionValue(fileName);
    if (tmpString != null) StringExpander.setFileName(tmpString);
    tmpString = (String)parser.getOptionValue(label);
    if (tmpString != null) StringExpander.setLabel(tmpString);
    tmpString = (String)parser.getOptionValue(genLabel);
    if (tmpString != null) StringExpander.setGenLabel(tmpString);
    tmpString = (String)parser.getOptionValue(genRef);
    if (tmpString != null) StringExpander.setGenRef(tmpString);
    tmpString = (String)parser.getOptionValue(outputType);
    if (tmpString != null) StringExpander.setProgLang(tmpString);

    Integer tmpInt;
    tmpInt = (Integer)parser.getOptionValue(beginLine);
    if (tmpInt != null) StringExpander.setBegin(tmpInt.intValue());
    tmpInt = (Integer)parser.getOptionValue(endLine);
    if (tmpInt != null) StringExpander.setEnd(tmpInt.intValue());
    
    // Extract the trailing command-line arguments.
    String[] otherArgs = parser.getRemainingArgs();
    if (otherArgs.length == 1) tmpFileName = otherArgs[0];
    else
    {
      printUsage();
      System.err.println("Error during command line parsing, exiting.");
      System.exit(2);
    }
  }
  
  /**
  Print the usage message.
  */
  static void printUsage()
  {
    System.out.println("PDwrap version 0.1. Copyright (c) 2004, Roland J. Weiss.\n");
    System.out.println("pdwrap <options> tmpfile");
    System.out.println(" <options> may be the following {short version, long version}:");
    System.out.println(" {c, CfgFile}: the configuration file");
    System.out.println(" {N, Name}: name of the original source file");
    System.out.println(" {L, Label}: user provided label");
    System.out.println(" {D, GenLabel}: ProgDoc generated label");
    System.out.println(" {R, GenRef}: ProgDoc generated reference");
    System.out.println(" {R, GenRef}: ProgDoc generated reference");
    System.out.println(" {b, begin}: start line of source listing");
    System.out.println(" {e, end}: end line of source listing");
    System.out.println(" {A, Caption}: user provided caption");
    System.out.println(" {m, ProgLang}: user provided output type of the listing");
    System.out.println(" {t, pdweaveCompat}: provided for compatibility with pdweave");
  }
  
  //----------------------------------------------------------------------------
  // Static data.

  // Hold the name of the xml configuration file.
  static String xmlFileName = new String();
  // Hold the name of the temporary file containing the listing.
  static String tmpFileName = new String();
}
